import { ManagedErrorType } from "./managed-error";
export type ExecuteManagedSyncFunc = (managedSyncFuncExecArgs: any) => any;
export type ExecuteManagedAsyncFunc = (managedAsyncFuncExecArgs: any) => Promise<any>;
export interface ManagedFuncDef<TManagedFuncName extends string, TExecuteManageFunc> {
    name: TManagedFuncName;
    namespace?: string;
    errorTypes: Array<ManagedErrorType<string>>;
}
export type ManagedSyncFuncDef<TManagedSyncFuncName extends string, TExecuteManagedSyncFunc extends ExecuteManagedSyncFunc> = ManagedFuncDef<TManagedSyncFuncName, TExecuteManagedSyncFunc>;
export type ManagedAsyncFuncDef<TManagedAsyncFuncName extends string, TExecuteManagedAsyncFunc extends ExecuteManagedAsyncFunc> = ManagedFuncDef<TManagedAsyncFuncName, TExecuteManagedAsyncFunc>;
export declare const defineManagedFunc: () => {
    name: <TManagedFuncName extends string>(managedFuncName: TManagedFuncName) => {
        namespace: (managedFuncNamespace?: string) => {
            errorTypes: (managedFuncErrorTypes: Array<ManagedErrorType<string>>) => {
                sync: () => {
                    executeType: <TExecuteManagedSyncFunc extends ExecuteManagedSyncFunc>() => { [ManagedSyncFuncName in TManagedFuncName as `${ManagedSyncFuncName}Def`]: ManagedSyncFuncDef<TManagedFuncName, TExecuteManagedSyncFunc>; };
                };
                async: () => {
                    executeType: <TExecuteManagedAsyncFunc extends ExecuteManagedAsyncFunc>() => { [ManagedAsyncFuncName in TManagedFuncName as `${ManagedAsyncFuncName}Def`]: ManagedAsyncFuncDef<TManagedFuncName, TExecuteManagedAsyncFunc>; };
                };
            };
        };
    };
};
export declare const getManagedFuncFullname: (managedFuncName: string, managedFuncNamespace?: string) => string;
export interface ManagedFuncConfig {
    traceLog: {
        isEnabled: boolean;
        isExecArgsIncluded: boolean;
        isExecResultIncluded: boolean;
    };
}
export type ManagedSyncFunc<TManagedSyncFuncName extends string, TExecuteSyncManagedFunc extends ExecuteManagedSyncFunc> = (TExecuteSyncManagedFunc & {
    /** @hidden */
    def: ManagedSyncFuncDef<TManagedSyncFuncName, TExecuteSyncManagedFunc>;
    /** @hidden */
    config: ManagedFuncConfig;
});
export type ManagedAsyncFunc<TManagedAsyncFuncName extends string, TExecuteManagedAsyncFunc extends ExecuteManagedAsyncFunc> = (TExecuteManagedAsyncFunc & {
    /** @hidden */
    def: ManagedAsyncFuncDef<TManagedAsyncFuncName, TExecuteManagedAsyncFunc>;
    /** @hidden */
    config: ManagedFuncConfig;
});
export declare const declareManagedFunc: () => {
    name: <TManagedFuncName extends string>(managedFuncName: TManagedFuncName) => {
        namespace: (managedFuncNamespace?: string) => {
            errorTypes: (managedFuncErrorTypes: Array<ManagedErrorType<string>>) => {
                sync: () => {
                    execute: <TExecuteManagedSyncFunc extends ExecuteManagedSyncFunc>(executeManagedSyncFunc: TExecuteManagedSyncFunc) => { [ManagedSyncFuncName in TManagedFuncName]: ManagedSyncFunc<TManagedFuncName, TExecuteManagedSyncFunc>; };
                };
                async: () => {
                    execute: <TExecuteManagedAsyncFunc extends ExecuteManagedAsyncFunc>(executeManagedAsyncFunc: TExecuteManagedAsyncFunc) => { [ManagedAsyncFuncName in TManagedFuncName]: ManagedAsyncFunc<TManagedFuncName, TExecuteManagedAsyncFunc>; };
                };
            };
        };
    };
    sync: () => {
        def: <TManagedSyncFuncName extends string, TExecuteManagedSyncFunc extends ExecuteManagedSyncFunc>(managedSyncFuncDef: ManagedSyncFuncDef<TManagedSyncFuncName, TExecuteManagedSyncFunc>) => {
            execute: (executeManagedSyncFunc: TExecuteManagedSyncFunc) => { [ManagedSyncFuncName in TManagedSyncFuncName]: ManagedSyncFunc<TManagedSyncFuncName, TExecuteManagedSyncFunc>; };
        };
    };
    async: () => {
        def: <TManagedAsyncFuncName extends string, TExecuteManagedAsyncFunc extends ExecuteManagedAsyncFunc>(managedAsyncFuncDef: ManagedAsyncFuncDef<TManagedAsyncFuncName, TExecuteManagedAsyncFunc>) => {
            execute: (executeManagedAsyncFunc: TExecuteManagedAsyncFunc) => { [ManagedAsyncFuncName in TManagedAsyncFuncName]: ManagedAsyncFunc<TManagedAsyncFuncName, TExecuteManagedAsyncFunc>; };
        };
    };
};
//# sourceMappingURL=managed-func.d.ts.map