"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SyncScheduler = exports.AsyncScheduler = void 0;
const exp_1 = require("./exp");
class AsyncScheduler {
    constructor({ jobMaxIntervalMs, executeJob, }) {
        this._jobMaxIntervalMs = jobMaxIntervalMs;
        this._executeJob = executeJob;
        this.triggerJob();
    }
    triggerJob() {
        (0, exp_1.invokeAndIgnoreAsyncExp)(async () => {
            const jobStartedDate = new Date();
            try {
                this._executeCurrentJobPromise = this._executeJob();
                await this._executeCurrentJobPromise;
            }
            catch (_a) {
            }
            const jobEndedDate = new Date();
            const msBetweenJobStartedAndJobEnded = jobEndedDate.getTime() - jobStartedDate.getTime();
            this._executeNextJobTimeoutId = setTimeout(() => {
                this.triggerJob();
            }, Math.max(0, this._jobMaxIntervalMs - msBetweenJobStartedAndJobEnded));
        });
    }
    async [Symbol.asyncDispose]() {
        try {
            if (this._executeNextJobTimeoutId !== undefined) {
                clearTimeout(this._executeNextJobTimeoutId);
            }
        }
        catch (_a) {
        }
        try {
            if (this._executeCurrentJobPromise !== undefined) {
                await this._executeCurrentJobPromise;
            }
        }
        catch (_b) {
        }
    }
}
exports.AsyncScheduler = AsyncScheduler;
class SyncScheduler {
    constructor({ jobMaxIntervalMs, executeJob, }) {
        this._jobMaxIntervalMs = jobMaxIntervalMs;
        this._executeJob = executeJob;
        this.triggerJob();
    }
    triggerJob() {
        try {
            const jobStartedDate = new Date();
            try {
                this._executeJob();
            }
            catch (_a) {
            }
            const jobEndedDate = new Date();
            const jobDurationMs = jobEndedDate.getTime() - jobStartedDate.getTime();
            this._executeNextJobTimeoutId = setTimeout(() => {
                this.triggerJob();
            }, Math.max(0, this._jobMaxIntervalMs - jobDurationMs));
        }
        catch (_b) {
        }
    }
    [Symbol.dispose]() {
        try {
            if (this._executeNextJobTimeoutId !== undefined) {
                clearTimeout(this._executeNextJobTimeoutId);
            }
        }
        catch (_a) {
        }
    }
}
exports.SyncScheduler = SyncScheduler;
//# sourceMappingURL=scheduler.js.map